@@ -21,6 +21,22 @@
 #include <gdk/gdk.h>
 #include <gtk/gtkplug.h>
 
+#include <config.h>
+#include <debug.h>
+#include <gaim.h>
+#include <core.h>
+#include <gtkutils.h>
+#include <gtkplugin.h>
+#include <gtkconv.h>
+#include <gtkdialogs.h>
+#include <gtkprefs.h>
+#include <blist.h>
+#include <gtkblist.h>
+#include <signals.h>
+#include <util.h>
+#include <version.h>
+#include <internal.h>
+
 #include "cryptproto.h"
 #include "cryptutil.h"
 #include "state.h"
@@ -29,25 +45,10 @@
 #include "nonce.h"
 #include "prefs.h"
 #include "config_ui.h"
-#include "blist.h"
-
-#include "src/gtkplugin.h"
-
-#include "config.h"
-#include "src/debug.h"
-#include "src/internal.h"
-#include "src/ui.h"
-#include "src/gaim.h"
-#include "src/core.h"
-#include "src/prefs.h"
-#include "src/blist.h"
-#include "src/gtkblist.h"
-#include "src/signals.h"
-#include "src/gtkconv.h"
-#include "src/gtkutils.h"
-#include "src/util.h"
+#include "ge_blist.h"
 
 #include "encrypt.h"
+#include "nls.h"
 
 #include <time.h>
 #include <sys/types.h>
@@ -72,6 +73,10 @@
 #include "win32dep.h"
 #endif
 
+
+/* from Gaim's internal.h, but it isn't critical that it is in sync: */
+/* #define BUF_LONG 4096 */
+
 G_MODULE_IMPORT GSList *gaim_accounts;
 G_MODULE_IMPORT guint im_options;
 
@@ -112,10 +117,10 @@ static void reap_all_sent_messages(GaimConversation*);
 static void reap_old_sent_messages(GaimConversation*);
 
 /* Function pointers exported to Gaim */
-static gboolean GE_got_msg(GaimAccount *, char **, char **, GaimConvImFlags flags, void *);
-static void GE_send_msg(GaimAccount *, char *, char **, void *);
-static void GE_new_conv(GaimConversation *, void *);
-static void GE_del_conv(GaimConversation *, void *);
+static gboolean GE_got_msg_cb(GaimAccount *, char **, char **, GaimConvImFlags flags, void *);
+static void GE_send_msg_cb(GaimAccount *, char *, char **, void *);
+static void GE_new_conv_cb(GaimConversation *, void *);
+static void GE_del_conv_cb(GaimConversation *, void *);
 
 static GHashTable *header_table, *footer_table, *notify_table;
 static gchar* header_default;
@@ -140,8 +145,8 @@ static void GE_send_key(GaimAccount *acct, const char *name, int asError, gchar
    GaimConversation *conv;
 
    int header_size, footer_size;
-   const gchar* header = g_hash_table_lookup(header_table, (gpointer)gaim_account_get_protocol(acct));
-   const gchar* footer = g_hash_table_lookup(footer_table, (gpointer)gaim_account_get_protocol(acct));
+   const gchar* header = g_hash_table_lookup(header_table, gaim_account_get_protocol_id(acct));
+   const gchar* footer = g_hash_table_lookup(footer_table, gaim_account_get_protocol_id(acct));
 
    if (!header) header = header_default;
    if (!footer) footer = "";
@@ -166,14 +171,14 @@ static void GE_send_key(GaimAccount *acct, const char *name, int asError, gchar
    if (asError) {
       if (msg_id) {
          sprintf(msg, "%s: ErrKey: Prot %s: Len %d:%sResend:%s:%s", header,
-                 pub_key->proto->name, key_str->len, key_str->str, msg_id, footer);
+                 pub_key->proto->name, (int)key_str->len, key_str->str, msg_id, footer);
       } else {
          sprintf(msg, "%s: ErrKey: Prot %s: Len %d:%s%s", header,
-                 pub_key->proto->name, key_str->len, key_str->str, footer);
+                 pub_key->proto->name, (int)key_str->len, key_str->str, footer);
       }
    } else {
       sprintf(msg, "%s: Key: Prot %s: Len %d:%s%s", header,
-              pub_key->proto->name, key_str->len, key_str->str, footer);
+              pub_key->proto->name, (int)key_str->len, key_str->str, footer);
    }
 
    if (strlen(msg) > GE_get_msg_size_limit(acct)) {
@@ -201,8 +206,8 @@ static crypt_key *GE_get_key(GaimConnection *gc, const char *name) {
    unsigned char* tmpmsg;
 
    int header_size, footer_size;
-   const gchar* header = g_hash_table_lookup(header_table, (gpointer)gaim_account_get_protocol(gc->account));
-   const gchar* footer = g_hash_table_lookup(footer_table, (gpointer)gaim_account_get_protocol(gc->account));
+   const gchar* header = g_hash_table_lookup(header_table, gaim_account_get_protocol_id(gc->account));
+   const gchar* footer = g_hash_table_lookup(footer_table, gaim_account_get_protocol_id(gc->account));
 
    if (!header) header = header_default;
    if (!footer) footer = "";
@@ -302,7 +307,7 @@ void GE_send_stored_msgs(GaimAccount* acct, const char* who) {
                  "Looking for stored msg:%s:%s\n",node->who, who);
       if ((strcmp(node->who, who) == 0) && (node->gc->account == acct)) {
          tmp_msg = g_strdup(node->msg);
-         GE_send_msg(node->gc->account, (char*)who, &tmp_msg, 0);
+         GE_send_msg_cb(node->gc->account, (char*)who, &tmp_msg, 0);
          GE_clear_string(node->msg);
          if (tmp_msg != 0) {
             g_free(tmp_msg);
@@ -456,21 +461,22 @@ static void reap_old_sent_messages(GaimConversation* conv){
    }
 }
 
-static gboolean GE_got_msg(GaimAccount *acct, char **who, char **message, GaimConvImFlags flags, void *m) {
+static gboolean GE_got_msg_cb(GaimAccount *acct, char **who, char **message, GaimConvImFlags flags, void *m) {
    char *name;
 
    GaimConversation *conv;
    gchar *headerpos;    /* Header is allowed to be anywhere in message now */
-   gchar *caps_header, *caps_message;    /* temps for ascii_strup() versions of each */
-                                         /* since Jabber mucks with case             */
+   gchar *notifypos;
+   gchar *caps_header, *caps_message,    /* temps for ascii_strup() versions of each */
+         *caps_notify;                   /* since Jabber mucks with case             */
 
    gchar *unescaped_message;             /* temps for html_unescaped       */
                                          /* since ICQ will now escape HTML */
 
    int header_size, footer_size;
-   const gchar* header = g_hash_table_lookup(header_table, (gpointer)gaim_account_get_protocol(acct));
-   const gchar* footer = g_hash_table_lookup(footer_table, (gpointer)gaim_account_get_protocol(acct));
-   const gchar* notify = g_hash_table_lookup(notify_table, (gpointer)gaim_account_get_protocol(acct));
+   const gchar* header = g_hash_table_lookup(header_table, gaim_account_get_protocol_id(acct));
+   const gchar* footer = g_hash_table_lookup(footer_table, gaim_account_get_protocol_id(acct));
+   const gchar* notify = g_hash_table_lookup(notify_table, gaim_account_get_protocol_id(acct));
 
    if (!header) header = header_default;
    if (!footer) footer = "";
@@ -481,7 +487,6 @@ static gboolean GE_got_msg(GaimAccount *acct, char **who, char **message, GaimCo
    /* Since we don't have a periodic callback, we do some housekeeping here */
    gaim_conversation_foreach(reap_old_sent_messages);
 
-
    conv = gaim_find_conversation_with_account(*who, acct);
    gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "Finding conversation: %s, %p\n", *who, conv);
    
@@ -495,6 +500,13 @@ static gboolean GE_got_msg(GaimAccount *acct, char **who, char **message, GaimCo
       headerpos = strstr(caps_message, caps_header);
       g_free(caps_header);
 
+      if (notify) {
+         caps_notify = g_ascii_strup(notify, -1);
+         notifypos = strstr(caps_message, caps_notify);
+         g_free(caps_notify);
+      } else {
+         notifypos = 0;
+      }
       if (headerpos != 0) {
          /* adjust to where the header is in the _real_ message, if */
          /* we found it in the caps_message                         */
@@ -503,13 +515,17 @@ static gboolean GE_got_msg(GaimAccount *acct, char **who, char **message, GaimCo
 
       g_free(caps_message);
 
-      if (headerpos == 0) {
+      if (headerpos == 0 && notifypos == 0) {
          /* Check for ICQ-escaped header*/
          headerpos = strstr(unescaped_message, header);
-         if ( (headerpos != 0) ||
-              ((notify != 0) && (strstr(unescaped_message, notify) != 0)) ) {
+         if (notify) {
+            notifypos = strstr(unescaped_message, notify);
+         }
+         if (headerpos != 0 || notifypos != 0) {
             /* ICQ PRPL escaped our HTML header, but we outsmarted it     */
             /* replace message with unescaped message.                    */
+            gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "Escaped header: replacing %s with %s\n",
+                       *message, unescaped_message);
             g_free(*message);
             *message = unescaped_message;
          } else {
@@ -560,7 +576,7 @@ static gboolean GE_got_msg(GaimAccount *acct, char **who, char **message, GaimCo
             gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
                        "Invalid Gaim-Encryption packet type\n"); 
          }            
-      } else if ((notify != 0) && (strstr(*message, notify) != 0)) {
+      } else if (notifypos != 0) {
          GE_set_rx_encryption(acct, name, FALSE);
          GE_set_capable(acct, name, TRUE);
          if (gaim_prefs_get_bool("/plugins/gtk/encrypt/encrypt_if_notified")) {
@@ -686,9 +702,11 @@ static void got_encrypted_msg(GaimConnection *gc, const char* name, char **messa
 /* Get account-specific message size limit*/
 
 static int GE_get_msg_size_limit(GaimAccount *acct) {
-   if (gaim_account_get_protocol(acct) == GAIM_PROTO_YAHOO) {
+   const char* protocol_id = gaim_account_get_protocol_id(acct);
+
+   if (strcmp(protocol_id, "prpl-yahoo") == 0) {
       return 945;
-   } else if (gaim_account_get_protocol(acct) == GAIM_PROTO_MSN) {
+   } else if (strcmp(protocol_id, "prpl-msn") == 0) {
       return 1500; /* This may be too small... somewhere in the 1500-1600 (+ html on front/back) */
    } else {
       /* Well, ok, this isn't too exciting.  Someday we can actually check  */
@@ -698,7 +716,7 @@ static int GE_get_msg_size_limit(GaimAccount *acct) {
    }
 }      
 
-static void GE_send_msg(GaimAccount *acct, char *who, char **message, void* data) {
+static void GE_send_msg_cb(GaimAccount *acct, char *who, char **message, void* data) {
    unsigned char *out_msg, *crypt_msg = 0;
    char *dupname;
    int msgsize;
@@ -713,9 +731,9 @@ static void GE_send_msg(GaimAccount *acct, char *who, char **message, void* data
    int baggage_size;
    char baggage[BUF_LONG];
 
-   const gchar* header = g_hash_table_lookup(header_table, (gpointer)gaim_account_get_protocol(acct));
-   const gchar* footer = g_hash_table_lookup(footer_table, (gpointer)gaim_account_get_protocol(acct));
-   const gchar* notify = g_hash_table_lookup(notify_table, (gpointer)gaim_account_get_protocol(acct));
+   const gchar* header = g_hash_table_lookup(header_table, gaim_account_get_protocol_id(acct));
+   const gchar* footer = g_hash_table_lookup(footer_table, gaim_account_get_protocol_id(acct));
+   const gchar* notify = g_hash_table_lookup(notify_table, gaim_account_get_protocol_id(acct));
 
    if (!header) header = header_default;
    if (!footer) footer = "";
@@ -867,8 +885,8 @@ void GE_resend_msg(GaimAccount* acct, const char* name, gchar *msg_id) {
       conv = gaim_conversation_new(GAIM_CONV_IM, acct, name);
    }
 
-   header = g_hash_table_lookup(header_table, (gpointer)gaim_account_get_protocol(conv->account));
-   footer = g_hash_table_lookup(footer_table, (gpointer)gaim_account_get_protocol(conv->account));
+   header = g_hash_table_lookup(header_table, gaim_account_get_protocol_id(conv->account));
+   footer = g_hash_table_lookup(footer_table, gaim_account_get_protocol_id(conv->account));
 
    if (!header) header = header_default;
    if (!footer) footer = "";
@@ -944,7 +962,9 @@ void GE_resend_msg(GaimAccount* acct, const char* name, gchar *msg_id) {
    } 
 }
 
-static void GE_new_conv(GaimConversation *conv, void* data) {
+
+
+static void GE_new_conv(GaimConversation *conv) {
    gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "New conversation\n");
 
    if ((conv != NULL) && (gaim_conversation_get_type(conv) == GAIM_CONV_IM)) {
@@ -956,7 +976,11 @@ static void GE_new_conv(GaimConversation *conv, void* data) {
    }
 }
 
-static void GE_del_conv(GaimConversation *conv, void* data)
+static void GE_new_conv_cb(GaimConversation *conv, void* data) {
+   GE_new_conv(conv);
+}
+
+static void GE_del_conv_cb(GaimConversation *conv, void* data)
 {
    GQueue *sent_msg_queue;
 
@@ -988,30 +1012,29 @@ static void GE_del_conv(GaimConversation *conv, void* data)
 }  
 
 static void GE_headers_init() {
-   header_table = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
-   footer_table = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
-   notify_table = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
+   header_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+   footer_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+   notify_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
-   g_hash_table_insert(header_table, (gpointer)GAIM_PROTO_TOC,
+   g_hash_table_insert(header_table, g_strdup("prpl-toc"),
                        g_strdup("*** Encrypted with the Gaim-Encryption plugin <A HREF=\""));
-   g_hash_table_insert(footer_table, (gpointer)GAIM_PROTO_TOC,
+   g_hash_table_insert(footer_table, g_strdup("prpl-toc"),
                        g_strdup("\"></A>"));
-   g_hash_table_insert(notify_table, (gpointer)GAIM_PROTO_TOC,
+   g_hash_table_insert(notify_table, g_strdup("prpl-toc"),
                        g_strdup("<A HREF=\"Gaim-Encryption Capable\"></A>"));
 
-
-   g_hash_table_insert(header_table, (gpointer)GAIM_PROTO_OSCAR,
+   g_hash_table_insert(header_table, g_strdup("prpl-oscar"),
                        g_strdup("*** Encrypted with the Gaim-Encryption plugin <A HREF=\""));
-   g_hash_table_insert(footer_table, (gpointer)GAIM_PROTO_OSCAR,
+   g_hash_table_insert(footer_table, g_strdup("prpl-oscar"),
                        g_strdup("\"></A>"));
-   g_hash_table_insert(notify_table, (gpointer)GAIM_PROTO_OSCAR,
+   g_hash_table_insert(notify_table, g_strdup("prpl-oscar"),
                        g_strdup("<A HREF=\"Gaim-Encryption Capable\"></A>"));
 
-   g_hash_table_insert(header_table, (gpointer)GAIM_PROTO_JABBER,
+   g_hash_table_insert(header_table, g_strdup("prpl-jabber"),
                        g_strdup("*** Encrypted with the Gaim-Encryption plugin <A HREF='"));
-   g_hash_table_insert(footer_table, (gpointer)GAIM_PROTO_JABBER,
+   g_hash_table_insert(footer_table, g_strdup("prpl-jabber"),
                        g_strdup("'></A>"));
-   g_hash_table_insert(notify_table, (gpointer)GAIM_PROTO_JABBER,
+   g_hash_table_insert(notify_table, g_strdup("prpl-jabber"),
                        g_strdup("<A HREF='Gaim-Encryption Capable'> </A>"));
 
    header_default = g_strdup("*** Encrypted :");
@@ -1060,12 +1083,19 @@ static gboolean GE_plugin_load(GaimPlugin *h) {
 
 	void *conv_handle;
 
+#ifdef ENABLE_NLS
+   bindtextdomain (ENC_PACKAGE, LOCALEDIR);
+   bind_textdomain_codeset (ENC_PACKAGE, "UTF-8");
+   setlocale(LC_ALL, "");
+#endif
+
    if (strcmp(gaim_core_get_version(), VERSION) != 0) {
-      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Compiled with '%s', running with '%s'."
-                 "  Gaim-Encryption will probably not run right.\n",
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption",
+                 _("Compiled with '%s', running with '%s'."
+                   "  Gaim-Encryption will probably not run right.\n"),
                  VERSION, gaim_core_get_version());
-      GE_error_window("Gaim-Encryption plugin was compiled with Gaim "
-                      VERSION ", not this version.");
+      GE_error_window(_("Gaim-Encryption plugin was compiled with a different "
+                        "version of Gaim.  You may experience problems."));
    }
 
    init_prefs();
@@ -1088,26 +1118,45 @@ static gboolean GE_plugin_load(GaimPlugin *h) {
    GE_headers_init();
 
    gaim_signal_connect(conv_handle, "receiving-im-msg", h, 
-                       GAIM_CALLBACK(GE_got_msg), NULL);
+                       GAIM_CALLBACK(GE_got_msg_cb), NULL);
    gaim_signal_connect(conv_handle, "sending-im-msg", h,
-                       GAIM_CALLBACK(GE_send_msg), NULL);
+                       GAIM_CALLBACK(GE_send_msg_cb), NULL);
    gaim_signal_connect(conv_handle, "conversation-created", h,
-                       GAIM_CALLBACK(GE_new_conv), NULL);
+                       GAIM_CALLBACK(GE_new_conv_cb), NULL);
    gaim_signal_connect(conv_handle, "deleting-conversation", h, 
-                       GAIM_CALLBACK(GE_del_conv), NULL);
+                       GAIM_CALLBACK(GE_del_conv_cb), NULL);
   
    gaim_signal_connect(gaim_blist_get_handle(), "blist-node-extended-menu", h,
-                       GAIM_CALLBACK(GE_buddy_menu), NULL);
+                       GAIM_CALLBACK(GE_buddy_menu_cb), NULL);
 
-   gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "done loading\n", gaim_get_blist());
+
+   gaim_conversation_foreach(GE_add_buttons);
+
+   gaim_debug(GAIM_DEBUG_MISC, "gaim-encryption", "done loading\n");
 
 
    return TRUE;
 }
 
 /* Called by Gaim when plugin is removed */
-static gboolean GE_plugin_unload() {
+static gboolean GE_plugin_unload(GaimPlugin *h) {
+   void * conv_handle = gaim_conversations_get_handle();
+
+   gaim_signal_disconnect(conv_handle, "receiving-im-msg", h, 
+                          GAIM_CALLBACK(GE_got_msg_cb));
+   gaim_signal_disconnect(conv_handle, "sending-im-msg", h,
+                          GAIM_CALLBACK(GE_send_msg_cb));
+   gaim_signal_disconnect(conv_handle, "conversation-created", h,
+                          GAIM_CALLBACK(GE_new_conv_cb));
+   gaim_signal_disconnect(conv_handle, "deleting-conversation", h, 
+                          GAIM_CALLBACK(GE_del_conv_cb));
+   
+   gaim_signal_disconnect(gaim_blist_get_handle(), "blist-node-extended-menu", h,
+                          GAIM_CALLBACK(GE_buddy_menu_cb));
+
    GE_config_unload();
+   
+   gaim_conversation_foreach(GE_remove_buttons);
 
    GE_my_priv_ring = GE_clear_ring(GE_my_priv_ring);
    GE_my_pub_ring = GE_clear_ring(GE_my_pub_ring);
@@ -1124,36 +1173,51 @@ static GaimGtkPluginUiInfo ui_info =
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
-	GAIM_PLUGIN_STANDARD,                             /**< type           */
-	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
-	0,                                                /**< flags          */
-	NULL,                                             /**< dependencies   */
-	GAIM_PRIORITY_DEFAULT,                            /**< priority       */
+   GAIM_PLUGIN_MAGIC,                                /**< I'm a plugin!  */
+   GAIM_MAJOR_VERSION,
+   GAIM_MINOR_VERSION,
+   GAIM_PLUGIN_STANDARD,                             /**< type           */
+   GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
+   0,                                                /**< flags          */
+   NULL,                                             /**< dependencies   */
+   GAIM_PRIORITY_DEFAULT,                            /**< priority       */
 
-	ENCRYPT_PLUGIN_ID,                                /**< id             */
-	N_("Gaim-Encryption"),                            /**< name           */
-   N_(ENC_VERSION " (compiled with " VERSION ")"),   /**< version        */
-	                                                  
-	N_("Encrypts conversations with RSA encryption."),/**  summary        */
+   ENCRYPT_PLUGIN_ID,                                /**< id             */
+   0,                                                /**< name           */
+   ENC_VERSION " (Gaim " VERSION ")",                /**< version        */
+   0,                                                /**  summary        */
+   0,                                                /**  description    */
+   0,                                                /**< author         */
+   ENC_WEBSITE,                                      /**< homepage       */
 
-	                                                  /**  description    */
-	N_("RSA encryption with keys up to 4096 bits,"
-      " using the Mozilla NSS crypto library.\n"),
-	N_("Bill Tompkins"),                              /**< author         */
-	ENC_WEBSITE,                                      /**< homepage       */
+   GE_plugin_load,                                   /**< load           */
+   GE_plugin_unload,                                 /**< unload         */
+   NULL,                                             /**< destroy        */
 
-	GE_plugin_load,                                   /**< load           */
-	GE_plugin_unload,                                 /**< unload         */
-	NULL,                                             /**< destroy        */
-
-	&ui_info,                                         /**< ui_info        */
-	NULL                                              /**< extra_info     */
+   &ui_info,                                         /**< ui_info        */
+   NULL,                                             /**< extra_info     */
+   NULL,                                             /**< prefs_info     */
+   NULL                                              /**< actions        */
 };
 
 static void
 init_plugin(GaimPlugin *plugin)
 {
+
+#ifdef ENABLE_NLS
+   bindtextdomain (ENC_PACKAGE, LOCALEDIR);
+   bind_textdomain_codeset (ENC_PACKAGE, "UTF-8");
+   setlocale(LC_ALL, "");
+#endif
+
+   info.name = _("Gaim-Encryption");
+   info.summary = _("Encrypts conversations with RSA encryption.");
+   info.description = _("RSA encryption with keys up to 4096 bits,"
+                        " using the Mozilla NSS crypto library.\n");
+   /* Translators: Feel free to add your name to the author field, with text like  */
+   /*   "Bill Tompkins, translation by Phil McGee"                                   */
+   info.author = _("Bill Tompkins");
+
 }
 
 
