@@ -21,18 +21,9 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkplug.h>
 
-#include "keys.h"
-#include "cryptutil.h"
-#include "prefs.h"
-#include "encrypt.h"
-#include "keys_ui.h"
-#include "ui.h"
-
-
-#include "src/debug.h"
-#include "src/internal.h"
-#include "src/gaim.h"
-#include "src/util.h"
+#include <debug.h>
+#include <gaim.h>
+#include <util.h>
 
 #include <time.h>
 #include <sys/types.h>
@@ -48,12 +39,20 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include "keys.h"
+#include "cryptutil.h"
+#include "prefs.h"
+#include "encrypt.h"
+#include "keys_ui.h"
+#include "ge_ui.h"
+#include "nls.h"
+
 #ifdef _WIN32
 #include "win32dep.h"
 #endif
 
 /* List of all the keys we know about */
-key_ring *GE_buddy_ring = 0, *GE_my_priv_ring = 0, *GE_my_pub_ring = 0;
+key_ring *GE_buddy_ring = 0, *GE_saved_buddy_ring = 0, *GE_my_priv_ring = 0, *GE_my_pub_ring = 0;
 
 typedef enum {KEY_MATCH, KEY_NOT_THERE, KEY_CONFLICT} KeyCheckVal;
 
@@ -221,8 +220,14 @@ void GE_received_key(char *key_msg, char *name, GaimAccount* acct, GaimConversat
    
    /* Make a new node for the linked list */
    new_key = g_malloc(sizeof(key_ring_data));
-   new_key->key = proto->parse_sent_key(key_len_msg);
    new_key->account = acct;
+   new_key->key = proto->parse_sent_key(key_len_msg);
+
+   if (new_key->key == 0) {
+      g_free(new_key);
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Invalid key received\n");
+      return;
+   }
 
    strncpy(new_key->name, name, sizeof(new_key->name));
    
@@ -265,7 +270,7 @@ void GE_received_key(char *key_msg, char *name, GaimAccount* acct, GaimConversat
 static KeyCheckVal GE_check_known_key(const char* filename, key_ring_data* key) {
    char line[MAX_KEY_STORLEN];
    GString *line_str, *key_str, *name_str;
-   char path[BUF_LONG];
+   char path[4096];
    
    struct stat fs;
    FILE* fp;
@@ -279,7 +284,7 @@ static KeyCheckVal GE_check_known_key(const char* filename, key_ring_data* key)
    /* check file permissions */
    if (stat(path, &fs) == -1) {
       /* file doesn't exist, so make it */
-      fd = open(path, O_CREAT | O_EXCL);
+      fd = open(path, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
       if (fd == -1) {
          /* Ok, maybe something else strange is going on... */
          gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error trying to create a known key file\n");
@@ -345,7 +350,7 @@ static KeyCheckVal GE_check_known_key(const char* filename, key_ring_data* key)
 void GE_add_key_to_file(const char *filename, key_ring_data* key) {
    GString *line_str, *key_str;
 
-   char path[BUF_LONG];
+   char path[4096];
    char errbuf[500];
    
    FILE* fp;
@@ -422,9 +427,109 @@ void GE_add_key_to_file(const char *filename, key_ring_data* key) {
    g_string_free(line_str, TRUE);
 }
 
-static void GE_del_key_from_file(const char *filename, const char *name, GaimAccount *acct) {
+void GE_del_one_key_from_file(const char *filename, int key_num, const char *name) {
    char line[MAX_KEY_STORLEN];
-   char path[BUF_LONG], tmp_path[BUF_LONG];
+   char path[4096], tmp_path[4096];
+   int foundit = 0;
+   FILE *fp, *tmp_fp;
+   int fd;
+   int line_num;
+
+   GString *line_start, *old_style_start, *normalized_start;
+
+   line_start = g_string_new(name);
+   GE_escape_name(line_start);
+   g_string_append_printf(line_start, ",");
+
+   old_style_start = g_string_new(name);
+   GE_escape_name(old_style_start);
+   g_string_append_printf(old_style_start, " ");
+
+   normalized_start = g_string_new(name);
+   GE_escape_name(normalized_start);
+   g_string_append_printf(normalized_start, " ");
+
+   g_snprintf(path, sizeof(path), "%s%s%s", gaim_home_dir(), G_DIR_SEPARATOR_S, filename);
+   
+   /* Look for name in the file.  If it's not there, we're done */
+   fp = fopen(path, "r");
+
+   if (fp == NULL) {
+      g_string_free(line_start, TRUE);
+      g_string_free(old_style_start, TRUE);
+      g_string_free(normalized_start, TRUE);
+      return;
+   }
+
+   for (line_num = 0; line_num <= key_num; ++line_num) {
+      fgets(line, sizeof(line), fp);
+   }
+
+   if ( (strncmp(line, line_start->str, line_start->len) == 0) ||
+        (strncmp(line, old_style_start->str, old_style_start->len) == 0) ||
+        (strncmp(line, normalized_start->str, normalized_start->len) == 0) ) {
+      foundit = 1;
+   }
+
+   fclose(fp);
+
+   gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Delete one key: found(%d)\n", foundit);
+
+   if (!foundit) {
+      g_string_free(line_start, TRUE);
+      g_string_free(old_style_start, TRUE);
+      g_string_free(normalized_start, TRUE);
+      return;
+   }
+
+   /* It's there.  Move file to a temporary, and copy the other lines */
+
+   g_snprintf(tmp_path, sizeof(tmp_path), "%s.tmp", path);
+   rename(path, tmp_path);
+   
+   fd = open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+   if (fd == -1) {
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Error opening key file %s\n", path);
+      perror("Error opening key file");
+      g_string_free(line_start, TRUE);
+      g_string_free(old_style_start, TRUE);
+      g_string_free(normalized_start, TRUE);
+
+      return;
+   }
+   fp = fdopen(fd, "a+");
+   
+   tmp_fp = fopen(tmp_path, "r");
+   if (tmp_fp == NULL) {
+      gaim_debug(GAIM_DEBUG_ERROR, "gaim-encryption", "Wah!  I moved a file and now it is gone\n");
+      fclose(fp);
+      g_string_free(line_start, TRUE);
+      g_string_free(old_style_start, TRUE);
+      g_string_free(normalized_start, TRUE);
+
+      return;
+   }
+
+   line_num = 0;
+   while (fgets(line, sizeof(line), tmp_fp)) {
+      if (line_num != key_num) {
+         fputs(line, fp);
+      } else {
+         gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "Skipping line %d\n", line_num);
+      }
+      ++line_num;
+   }
+   
+   fclose(fp);
+   fclose(tmp_fp);
+   unlink(tmp_path);
+   g_string_free(line_start, TRUE);
+}
+
+
+void GE_del_key_from_file(const char *filename, const char *name, GaimAccount *acct) {
+   char line[MAX_KEY_STORLEN];
+   char path[4096], tmp_path[4096];
    int foundit = 0;
    FILE *fp, *tmp_fp;
    int fd;
@@ -432,7 +537,11 @@ static void GE_del_key_from_file(const char *filename, const char *name, GaimAcc
 
    line_start = g_string_new(name);
    GE_escape_name(line_start);
-   g_string_append_printf(line_start, ",%s", gaim_account_get_protocol_id(acct));
+   if (acct != 0) {
+      g_string_append_printf(line_start, ",%s", gaim_account_get_protocol_id(acct));
+   } else {
+      g_string_append_printf(line_start, ",");
+   }
 
    old_style_start = g_string_new(name);
    GE_escape_name(old_style_start);
@@ -515,7 +624,7 @@ static void GE_del_key_from_file(const char *filename, const char *name, GaimAcc
 key_ring * GE_load_keys(const char *filename) {
    FILE* fp;
    char name[64], nameacct[164], proto[20], proto_name[10], proto_ver[10], key_str_buf[MAX_KEY_STORLEN];
-   char path[BUF_LONG];
+   char path[4096];
    int rv;
    key_ring *new_ring = 0;
    key_ring_data *new_key;
@@ -544,8 +653,10 @@ key_ring * GE_load_keys(const char *filename) {
             /* This will do the right thing: if no account, it will match any */
             account = gaim_accounts_find(name, nameaccount_split[1]);
 
-            /* gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "load_keys: account: %p\n", account);           */
-            /* gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "%p\n", gaim_account_get_protocol_id(account)); */
+            gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "load_keys: name(%s), protocol (%s): %p\n",
+                       nameaccount_split[0], nameaccount_split[1], account);
+
+            gaim_debug(GAIM_DEBUG_INFO, "gaim-encryption", "%p\n", gaim_account_get_protocol_id(account));
 
             g_strfreev(nameaccount_split);
 
@@ -584,8 +695,13 @@ key_ring * GE_load_keys(const char *filename) {
 
       fclose(fp);
    } else {
-      gaim_debug(GAIM_DEBUG_WARNING, "gaim-encryption", "Couldn't open file:%s\n", path);
-      perror("Error opening file");
+      if (errno != ENOENT) {
+         gaim_debug(GAIM_DEBUG_WARNING, "gaim-encryption", "Couldn't open file:%s\n", path);
+         perror("Error opening file");
+      } else {
+         gaim_debug(GAIM_DEBUG_WARNING, "gaim-encryption",
+                    "File %s doesn't exist (yet).  A new one will be created.\n", path);
+      }
    }
 
    return new_ring;
@@ -602,6 +718,7 @@ void GE_key_rings_init() {
    /* Load the keys */
    GE_my_pub_ring = GE_load_keys(Public_key_file);
    GE_my_priv_ring = GE_load_keys(Private_key_file);
+   GE_saved_buddy_ring = GE_load_keys(Buddy_key_file);
 
    /* Create a key for each screen name if we don't already have one */
    
